'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = dailyMetrics;

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _d = require('d3');

var d3 = _interopRequireWildcard(_d);


function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var margin = { top: 50, right: 20, bottom: 30, left: 20 };

function dailyMetrics(elem) {
  var props = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var svg = d3.select(elem).append('svg').attr('class', 'historical-counts');

  var g = svg.append("g").attr("transform", "translate(" + margin.left + "," + margin.top + ")");

  var xAxisGroup = g.append("g").attr("class", "daily-metrics-axis daily-metrics-axis-x");

  return function (_ref) {
    var data = _ref.data,
        width = _ref.width,
        height = _ref.height;

    svg.attr('height', height).attr('width', width).attr('viewBox', '0 0 ' + width + ' ' + height);

    var graphWidth = width - margin.left - margin.right;
    var graphHeight = height - margin.top - margin.bottom;

    //
    // Scales
    //

    var xScale = d3.scaleBand().domain(data.map(function (d) {
      return d.label;
    })).rangeRound([0, graphWidth]).padding(0.1);
    var yScale = d3.scaleLinear().domain([0, d3.max(data, function (d) {
      return d.value;
    })]).rangeRound([graphHeight, 0]);

    //
    // Axes
    //

    xAxisGroup.attr("transform", "translate(0," + graphHeight + ")").call(d3.axisBottom(xScale));

    var dataSelection = g.selectAll(".daily-metrics-bar-group").data(data);

    //
    // Enter
    //

    var dataSelectionEnter = dataSelection.enter();
    var dataSelectionEnterGroup = dataSelectionEnter.append('g').attr('class', 'daily-metrics-bar-group');
    dataSelectionEnterGroup.append('rect').attr('class', 'daily-metrics-bar').attr('x', 0).attr('y', 0);
    dataSelectionEnterGroup.append('path').attr('class', 'daily-metrics-bar-outline');
    dataSelectionEnterGroup.append('text').attr('class', 'daily-metrics-bar-label').attr('text-anchor', 'middle');

    //
    // Merge
    //
    var dataSelectionMerge = dataSelectionEnter.merge(dataSelection);
    dataSelectionMerge.selectAll('.daily-metrics-bar-group').attr('transform', function (d) {
      return 'translate(' + xScale(d.label) + ',' + yScale(d.value) + ')';
    });
    dataSelectionMerge.selectAll('.daily-metrics-bar').attr('width', xScale.bandwidth()).attr('height', function (d) {
      return graphHeight - yScale(d.value);
    });
    dataSelectionMerge.selectAll('.daily-metrics-bar-outline').attr('d', function (d) {
      return 'M0,' + (graphHeight - yScale(d.value)) + ' V0 H' + xScale.bandwidth() + ' V' + (graphHeight - yScale(d.value));
    });
    dataSelectionMerge.selectAll('.daily-metrics-bar-label').attr('transform', function (d) {
      return 'translate(' + xScale.bandwidth() / 2 + ',-10)';
    }).text(function (d) {
      return d.value;
    });
  };
}

