'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.exampleAxis = exampleAxis;
exports.xAxisDailyTick = xAxisDailyTick;
exports.yAxisMinMax = yAxisMinMax;

var _d = require('d3');

var d3 = _interopRequireWildcard(_d);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _colors = require('@density/ui/variables/colors.json');

var _colors2 = _interopRequireDefault(_colors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var ONE_MINUTE_IN_MS = 60 * 1000,
    ONE_HOUR_IN_MS = ONE_MINUTE_IN_MS * 60,
    ONE_DAY_IN_MS = ONE_HOUR_IN_MS * 60;

// A "simple" example axis. This demonstrates how to build an axis that works with the line chart.
// Usage as a react prop: xAxis={exampleAxis({color: 'red'})}
//
function exampleAxis(_ref) {
  var color = _ref.color;

  color = color || 'royalblue';
  return function (_ref2, element) {
    var scale = _ref2.scale;

    // Add an axis:
    element.call(d3.axisBottom(scale));

    // Do some custom drawing:
    var selection = element.selectAll('rect').data([10, 20, 30]);
    selection.enter().append('rect').attr('width', 10).attr('height', 10).attr('fill', color).merge(selection).attr('x', function (d) {
      return d;
    }).attr('y', function (d) {
      return d;
    });
    selection.exit().remove();
  };
}

function xAxisDailyTick(_ref3) {
  var formatter = _ref3.formatter,
      tickValues = _ref3.tickValues,
      tickResolutionInMs = _ref3.tickResolutionInMs,
      bottomOffset = _ref3.bottomOffset;

  bottomOffset = bottomOffset || 15;

  return function (_ref4, element) {
    var timeZone = _ref4.timeZone,
        scale = _ref4.scale,
        bottomMargin = _ref4.bottomMargin,
        dataPoints = _ref4.dataPoints;

    var axisFontSize = 14;

    // If manual tick values weren't specified, then generate our own using the `tickResolutionInMs`
    // property.
    tickValues = tickValues || d3.range(dataPoints.startXValue, dataPoints.endXValue, tickResolutionInMs || 1 * ONE_HOUR_IN_MS);

    var axis = d3.axisBottom(scale).tickValues(tickValues)

    // Position a tick at the first whole hour on the axis, then another tick for each hour
    // thereafter.
    .tickFormat(formatter || function (n) {
      // "5a" or "8p"
      var timeFormat = _moment2.default.utc(n).format('hA');
      return timeFormat.slice(0, timeFormat.startsWith('12') ? -1 : -2).toLowerCase();
    });

    // Render the axis
    element.call(axis);

    // Remove some parts of the axis that aren't needed
    element.select('.domain').remove();
    element.selectAll('.tick line').remove();

    // Adjust the vertical spacing of each tick on the axis
    element.selectAll('.tick text').attr('transform', 'translate(0,' + (-10 + bottomOffset) + ')').attr('font-size', axisFontSize);
  };
}

function yAxisMinMax(_ref5) {
  var formatter = _ref5.formatter,
      leftOffset = _ref5.leftOffset,
      showMinimumPoint = _ref5.showMinimumPoint,
      showMaximumPoint = _ref5.showMaximumPoint,
      points = _ref5.points,
      axisRuleLineDashWidth = _ref5.axisRuleLineDashWidth,
      axisRuleLineDashSpacing = _ref5.axisRuleLineDashSpacing;

  leftOffset = typeof leftOffset === 'undefined' ? 20 : leftOffset;
  axisRuleLineDashWidth = typeof axisRuleLineDashWidth === 'undefined' ? 4 : axisRuleLineDashWidth;
  axisRuleLineDashSpacing = typeof axisRuleLineDashSpacing === 'undefined' ? 10 : axisRuleLineDashSpacing;

  points = points || [];

  /* both showMinimumPoint and showMaximumPoint default to true if unspecified */
  showMinimumPoint = typeof showMinimumPoint === 'undefined' ? true : showMinimumPoint;
  showMaximumPoint = typeof showMaximumPoint === 'undefined' ? true : showMaximumPoint;

  return function (_ref6, element) {
    var graphWidth = _ref6.graphWidth,
        leftMargin = _ref6.leftMargin,
        firstEventYValue = _ref6.firstEventYValue,
        lastEventYValue = _ref6.lastEventYValue,
        scale = _ref6.scale,
        defaultDataset = _ref6.defaultDataset;

    var axisFontSize = 14;
    var axisPoints = [].concat(_toConsumableArray(showMinimumPoint ? [{ value: firstEventYValue, hasRule: false, hasShadow: false }] : []), _toConsumableArray(points || []), _toConsumableArray(showMaximumPoint ? [{ value: lastEventYValue, hasRule: false, hasShadow: false }] : []));

    var selection = element.selectAll('.axis-y-point').data(axisPoints, function (d) {
      return JSON.stringify(d);
    });

    // On enter selection, create a group with a text element and line inside. These will be used to
    // render each datapoint on the y axis.
    var enterSelection = selection.enter();
    var enterSelectionGroup = enterSelection.append('g').attr('class', 'axis-y-point').attr('font-size', axisFontSize);
    enterSelectionGroup.append('text').attr('class', 'axis-y-point-label');
    enterSelectionGroup.append('path').attr('class', 'axis-y-point-rule');
    enterSelectionGroup.append('rect').attr('class', 'axis-y-point-shadow');

    // On merge, add the text to each datapoint on the axis.
    var mergeSelection = enterSelectionGroup.merge(enterSelection).attr('transform', function (d) {
      var x = -1 * (leftMargin - leftOffset);
      var y = scale(d.value) + axisFontSize * 0.4;

      // Adjust the first datapoint to match with the vertical baseline of the chart.
      if (d.value === firstEventYValue) {
        y -= defaultDataset.verticalBaselineOffset || 0;
      }
      return 'translate(' + x + ',' + y + ')';
    });
    mergeSelection.select('.axis-y-point-label').text(formatter || function (d) {
      return '' + d.value;
    });
    mergeSelection.select('.axis-y-point-rule').attr('transform', 'translate(' + (leftMargin - leftOffset) + ',' + -1 * (axisFontSize * 0.4) + ')').attr('stroke', _colors2.default.gray).attr('stroke-width', '1').attr('shape-rendering', 'crispEdges').attr('d', function (d) {
      // Don't add horizontal lines for points that don't specify a rule is required.
      if (!d.hasRule) {
        return '';
      }

      return 'M0,0 ' + function (graphWidth) {
        var linePath = '';
        for (var i = 0; i < graphWidth; i += axisRuleLineDashWidth + axisRuleLineDashSpacing) {
          linePath += 'H' + (i + axisRuleLineDashWidth) + ' M' + (i + axisRuleLineDashWidth + axisRuleLineDashSpacing) + ',0';
        }
        return linePath;
      }(graphWidth);
    });
    mergeSelection.select('.axis-y-point-shadow').attr('x', leftMargin - leftOffset).attr('y', -1 * (axisFontSize * 0.4)).attr('width', graphWidth).attr('height', function (d) {
      return firstEventYValue - scale(d.value);
    }).attr('fill', function (d) {
      if (d.hasShadow) {
        return 'rgba(65, 152, 255, 0.1)';
      } else {
        return 'transparent';
      }
    });

    // On exit, remove any axis points that are no longer required
    selection.exit().remove();

    return element;
  };
}

