'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.overlayTwoPopups = overlayTwoPopups;
exports.overlayTwoPopupsPlainTextFormatter = overlayTwoPopupsPlainTextFormatter;
exports.overlayTwoPopupsPersonIconTextFormatter = overlayTwoPopupsPersonIconTextFormatter;
exports.overlayExample = overlayExample;

var _d = require('d3');

var d3 = _interopRequireWildcard(_d);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

require('moment-timezone');

var _colors = require('@density/ui/variables/colors.json');

var _colors2 = _interopRequireDefault(_colors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function overlayTwoPopups(_ref) {
  var topPopupFormatter = _ref.topPopupFormatter,
      bottomPopupFormatter = _ref.bottomPopupFormatter,
      bottomOverlayTopMargin = _ref.bottomOverlayTopMargin,
      topOverlayBottomMargin = _ref.topOverlayBottomMargin,
      topOverlayWidth = _ref.topOverlayWidth,
      topOverlayHeight = _ref.topOverlayHeight,
      bottomOverlayWidth = _ref.bottomOverlayWidth,
      bottomOverlayHeight = _ref.bottomOverlayHeight;

  return {
    enter: function enter(_ref2, selection) {
      var xScale = _ref2.xScale,
          yScale = _ref2.yScale;

      // Add filter for use in making shadows behind things.
      var filter = selection.append('filter').attr('id', 'two-popup-overlay-shadow');
      filter.append('feGaussianBlur').attr("in", "SourceAlpha").attr("stdDeviation", 5).attr("result", "blur");
      filter.append('feOffset').attr('dx', 2).attr('dy', 2).attr("result", "offsetBlur");
      var feMerge = filter.append("feMerge");
      feMerge.append("feMergeNode").attr("in", "offsetBlur");
      feMerge.append("feMergeNode").attr("in", "SourceGraphic");

      // Render a vertical line that indicates where the user is within the graph
      selection.append('line').attr('stroke', _colors2.default.brandPrimary).attr('shape-rendering', 'crispEdges');

      // Scaffold the top overlay
      var topOverlay = selection.append('g').attr('class', 'top-overlay');
      topOverlay.append('rect').attr('class', 'top-overlay-background shadow').attr('height', topOverlayHeight).attr('fill', 'rgba(0, 0, 0, 0.1)').style("filter", "url(#two-popup-overlay-shadow)");
      topOverlay.append('rect').attr('class', 'top-overlay-background').attr('fill', '#fff').attr('stroke', _colors2.default.gray).attr('stroke-width', 1).attr('rx', 4).attr('ry', 4).attr('height', topOverlayHeight);

      // Allow custom formatter code to run on the top overlay.
      var topOverlayLabel = topOverlay.append('g').attr('class', 'top-overlay-label');
      topPopupFormatter.enter(topOverlayLabel);

      // And scaffold the bottom overlay
      var bottomOverlay = selection.append('g').attr('class', 'bottom-overlay');
      bottomOverlay.append('rect').attr('class', 'bottom-overlay-background shadow').attr('height', bottomOverlayHeight).attr('fill', 'rgba(0, 0, 0, 0.1)').style("filter", "url(#two-popup-overlay-shadow)");
      bottomOverlay.append('rect').attr('class', 'bottom-overlay-background').attr('fill', '#fff').attr('stroke', _colors2.default.gray).attr('stroke-width', 1).attr('rx', 4).attr('ry', 4).attr('height', bottomOverlayHeight);

      // Allow custom formatter code to run on the top overlay.
      var bottomOverlayLabel = bottomOverlay.append('g').attr('class', 'bottom-overlay-label');
      bottomPopupFormatter.enter(bottomOverlayLabel);
    },
    merge: function merge(data, selection) {
      var mouseX = data.mouseX,
          mouseY = data.mouseY,
          xScale = data.xScale,
          yScale = data.yScale,
          graphWidth = data.graphWidth,
          graphHeight = data.graphHeight,
          defaultDataset = data.defaultDataset,
          dataPoints = data.dataPoints;

      var xInMs = xScale.invert(mouseX);

      // Figure out the event at the user's current mouse position
      var eventIndexAtOverlayPosition = d3.bisector(function (d) {
        return _moment2.default.utc(d.timestamp).valueOf();
      }).right(defaultDataset.data, xInMs) - 1;
      var eventAtPosition = defaultDataset.data[eventIndexAtOverlayPosition];

      // Ensure that the overlay is only being drawn when the user is hovering over the data
      //
      var renderAfterLastDatapoint = typeof defaultDataset.renderAfterLastDatapoint === 'undefined' ? false : true;
      if (!(eventAtPosition && (renderAfterLastDatapoint ? true : xInMs <= dataPoints.lastEventXValue))) {
        selection.remove();
        return;
      }

      var formatterData = _extends({}, data, {
        item: eventAtPosition,
        topOverlayWidth: topOverlayWidth,
        topOverlayHeight: topOverlayHeight,
        bottomOverlayWidth: bottomOverlayWidth,
        bottomOverlayHeight: bottomOverlayHeight
      });

      // Render a vertical line that indicates where the user is within the graph
      selection.select('line').attr('x1', mouseX).attr('y1', 0 - topOverlayBottomMargin).attr('x2', mouseX).attr('y2', graphHeight + bottomOverlayTopMargin);

      var topOverlayXPosition = mouseX - topOverlayWidth / 2;
      // Ensure that the top overlay won't underflow the svg
      if (topOverlayXPosition < 0) {
        topOverlayXPosition = 0;
      }
      // Ensure that the top overlay won't overflow the svg
      if (topOverlayXPosition + topOverlayWidth > graphWidth) {
        topOverlayXPosition = graphWidth - topOverlayWidth;
      }

      // Apply the new x, y, and width to the top overlay
      selection.selectAll('.top-overlay-background').attr('x', topOverlayXPosition).attr('y', 0 - topOverlayBottomMargin - topOverlayHeight).attr('width', topOverlayWidth);

      // Allow custom formatter code to run on the top overlay.
      var topOverlayLabel = selection.select('.top-overlay-label');
      topOverlayLabel.attr('transform', 'translate(' + topOverlayXPosition + ',' + (0 - topOverlayBottomMargin - topOverlayHeight) + ')');
      topPopupFormatter.merge(formatterData, topOverlayLabel);

      var bottomOverlayXPosition = mouseX - bottomOverlayWidth / 2;
      // Ensure that the bottom overlay won't underflow the svg
      if (bottomOverlayXPosition < 0) {
        bottomOverlayXPosition = 0;
      }
      // Ensure that the bottom overlay won't overflow the svg
      if (bottomOverlayXPosition + bottomOverlayWidth > graphWidth) {
        bottomOverlayXPosition = graphWidth - bottomOverlayWidth;
      }

      // Apply the new x, y, and width to the bottom overlay
      selection.selectAll('.bottom-overlay-background').attr('x', bottomOverlayXPosition).attr('y', graphHeight + bottomOverlayTopMargin).attr('width', bottomOverlayWidth);

      // Allow custom formatter code to run on the top overlay.
      var bottomOverlayLabel = selection.select('.bottom-overlay-label');
      bottomOverlayLabel.attr('transform', 'translate(' + bottomOverlayXPosition + ',' + (graphHeight + bottomOverlayTopMargin) + ')');
      bottomPopupFormatter.merge(formatterData, bottomOverlayLabel);
    },
    exit: function exit(props, selection) {
      selection.remove();
    }
  };
}

// Can be passed to the `formatter` prop of the `overlayTwoPopups` overlay.
// Renders the value as plain text, optionally running it through a mapping function first.
function overlayTwoPopupsPlainTextFormatter(mapping) {
  var position = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'bottom';

  return {
    enter: function enter(selection) {
      selection.append('text').attr('text-anchor', 'middle').attr('font-weight', '500');
    },
    merge: function merge(data, selection) {
      var item = data.item,
          xScale = data.xScale,
          mouseX = data.mouseX,
          topOverlayWidth = data.topOverlayWidth;

      var width = position === 'bottom' ? data.bottomOverlayWidth : data.topOverlayWidth;
      selection.select('text').attr('transform', 'translate(' + width / 2 + ',26)').text(mapping ? mapping(item, data) : item.value);
    },
    exit: function exit(selection) {
      return selection.remove();
    }
  };
}

// Can be passed to the `formatter` prop of the `overlayTwoPopups` overlay.
// Renders the value as plain text, but with a person icon to the left of the plain text, optionally
// running it through a mapping function first.
function overlayTwoPopupsPersonIconTextFormatter(mapping) {
  return {
    enter: function enter(selection) {
      selection.append('text').attr('text-anchor', 'middle').attr('font-weight', '500');

      // Created via https://rawgit.com/jColeChanged/svg2d3js/master/index.html
      var qv7a5d925c094c53a70 = selection.append('g').attr('stroke', 'none').attr('stroke-width', '1').attr('fill', 'none').attr('fill-rule', 'evenodd').attr('stroke-linejoin', 'round').attr('transform', 'translate(15,10)');
      var qvf0621598a7e8cb863 = qv7a5d925c094c53a70.append('g').attr('transform', 'translate(-775.000000, -397.000000)').attr('stroke', '#4198FF').attr('stroke-width', '1.5');
      var qvd28faf862aaae21da = qvf0621598a7e8cb863.append('g').attr('transform', 'translate(170.000000, 159.000000)');
      var qvf4650fd1e11506470 = qvd28faf862aaae21da.append('g').attr('transform', 'translate(591.000000, 227.000000)');
      var qv1fd943a9889237046 = qvf4650fd1e11506470.append('g').attr('transform', 'translate(15.000000, 12.000000)');
      var qv6fd596351c7e83872 = qv1fd943a9889237046.append('path').attr('d', 'M9.42856667,4 C9.42856667,6.20911111 7.89356667,8 6.00001111,8 C4.10645556,8 2.57145556,6.20911111 2.57145556,4 C2.57145556,1.79088889 4.10645556,0 6.00001111,0 C7.89356667,0 9.42856667,1.79088889 9.42856667,4 L9.42856667,4 Z');
      var qv79a38c2da85ee301f = qv1fd943a9889237046.append('path').attr('d', 'M12,16 L12,11 C12,9.33688889 10.6392222,8 8.97611111,8 L3.02388889,8 C1.36077778,8 0,9.33688889 0,11 L0,16 L12,16 L12,16 Z');
      var qvd5bb95460496d63c4 = qv1fd943a9889237046.append('path').attr('d', 'M3,12 L3,16');
      var qv3ba41bdbddbf151c1 = qv1fd943a9889237046.append('path').attr('d', 'M9,12 L9,16');
    },
    merge: function merge(data, selection) {
      var item = data.item,
          xScale = data.xScale,
          mouseX = data.mouseX,
          topOverlayWidth = data.topOverlayWidth;

      selection.select('text').attr('transform', 'translate(' + (24 + (topOverlayWidth - 24) / 2) + ',27)').text(mapping ? mapping(item, data) : item.value);
    },
    exit: function exit(selection) {
      return selection.remove();
    }
  };
}

function overlayExample(_ref3) {
  var color = _ref3.color;

  return {
    enter: function enter(data, selection) {
      selection.append('rect').attr('width', 10).attr('height', 10).attr('fill', color);
    },
    merge: function merge(_ref4, selection) {
      var mouseX = _ref4.mouseX,
          mouseY = _ref4.mouseY;

      selection.select('rect').attr('x', mouseX).attr('y', mouseY);
    },
    exit: function exit(props, selection) {
      selection.remove();
    }
  };
}

