'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.getIndicatorLocations = getIndicatorLocations;
exports.default = ingressEgress;

var _d = require('d3');

var d3 = _interopRequireWildcard(_d);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);


function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var SECONDS_PER_MINUTE = 60,
    MILLISECONDS_PER_SECOND = 1000;

// ------------------------------------------------------------------------------
// Constants required for rendering the graph
// ------------------------------------------------------------------------------
var eventMarkerRadiusDefault = 3; // The event marker radius in px
var eventMarkerSpacingFromMidLineDefault = 5; // Spacing between the event marker and the middle line
var eventMarkerInfoPopupHeightDefault = 20; // Height of the info popup, not including the pointer.
var eventMarkerInfoPopupWidthDefault = 20; // Width of the info popup that shows if more than one event happened.
var eventMarkerInfoPopupSpacingFromMarkerDefault = 10; // Spacing between the info popup and the event marker.
var eventMarkerInfoPopupCaretWidthDefault = 5;
var eventMarkerInfoPopupFontSizeDefault = 12;
var cardHeightInPx = 160;
var graphDurationInMin = 1;

// The Sailec font doesn't have a properly aligned baseline! Offset it manually :(
var sailecBaselineOffset = 3;

// Generate the proper time label for each fraction of the total time
function getTimeLabel(graphDurationInMin, fraction) {
  return graphDurationInMin * fraction >= 1 ? graphDurationInMin * fraction + 'm' : Math.floor(graphDurationInMin * fraction * 60) + 's';
};

function getIndicatorLocations(data) {
  var minimumStackDistance = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1500;

  var lastTimestamp = {};
  lastTimestamp[1] = 0;
  lastTimestamp[-1] = 0;

  return data.reduce(function (acc, i) {
    // Ensure the timestamp is an epoch.
    var timestamp = i.timestamp;
    if (typeof timestamp !== 'number') {
      if (timestamp instanceof _moment2.default) {
        timestamp = timestamp.valueOf();
      } else {
        timestamp = _moment2.default.utc(timestamp).valueOf();
      }
    }

    if (lastTimestamp[i.countChange] === 0 || timestamp - lastTimestamp[i.countChange] > minimumStackDistance) {
      // No previous event marker was found to stack the current count into. Create a new event marker.
      lastTimestamp[i.countChange] = timestamp;
      return [].concat(_toConsumableArray(acc), [{ timestamp: timestamp, count: i.countChange }]);
    } else {
      // In this case, a previous marker was found within the minimum stack distance.  Add this
      // event's count change to the count of the previous event, effectively squashing them.

      // Loop through all markers in reverse order, looking for the most recent marker that is on
      // the same side of the axis as this marker.
      var lastIndexOfIndicatorDirection = -1;
      for (var ct = acc.length - 1; ct >= 0; ct--) {
        if (i.countChange > 0 && acc[ct].count > 0 || i.countChange < 0 && acc[ct].count < 0) {
          lastIndexOfIndicatorDirection = ct;
          break;
        }
      }

      // Update the marker that was found with an updated count from the current marker.
      acc.splice(lastIndexOfIndicatorDirection, 1, _extends({}, acc[lastIndexOfIndicatorDirection], {
        count: acc[lastIndexOfIndicatorDirection].count + i.countChange
      }));

      lastTimestamp[i.countChange] = timestamp;
      return acc;
    }
  }, []);
}

// Generate the proper time label for each fraction of the total time
function ingressEgress(elem) {
  var card = d3.select(elem).append('div').attr('class', 'card card-dark ingress-egress-card');

  // add legend to card to explain what stuff means
  var legend = card.append('span').attr('class', 'real-time-capacity-legend');

  legend.append('div').attr('class', 'real-time-capacity-count-marker in').attr('style', 'width: ' + eventMarkerRadiusDefault * 2 + 'px; height: ' + eventMarkerRadiusDefault * 2 + 'px');
  legend.append('span').text('In');

  legend.append('div').attr('class', 'real-time-capacity-count-marker out').attr('style', 'width: ' + eventMarkerRadiusDefault * 2 + 'px; height: ' + eventMarkerRadiusDefault * 2 + 'px');
  legend.append('span').text('Out');

  // Svg to put all the data in
  var svg = card.append('svg').attr('class', 'ingress-egress-data').attr('width', '100%').attr('height', '100%');

  // Render the line down the middle
  var midLine = svg.append('line').attr('class', 'real-time-capacity-mid-line').attr('x1', '0').attr('y1', '50%').attr('x2', '100%').attr('y2', '50%');

  var dataGroup = svg.append('g').attr('class', 'ingress-egress-data');

  // Labels at the bottom of the card
  var labels = card.append('ul').attr('class', 'real-time-capacity-labels');
  labels.append('li').attr('class', 'real-time-capacity-labels-item').text(getTimeLabel(graphDurationInMin, 1) + ' ago');
  labels.append('li').attr('class', 'real-time-capacity-labels-item').text('Now');

  return function (_ref) {
    var events = _ref.events,
        eventMarkerRadius = _ref.eventMarkerRadius,
        eventMarkerSpacingFromMidLine = _ref.eventMarkerSpacingFromMidLine,
        eventMarkerInfoPopupWidth = _ref.eventMarkerInfoPopupWidth,
        eventMarkerInfoPopupHeight = _ref.eventMarkerInfoPopupHeight,
        eventMarkerInfoPopupSpacingFromMarker = _ref.eventMarkerInfoPopupSpacingFromMarker,
        eventMarkerInfoPopupCaretWidth = _ref.eventMarkerInfoPopupCaretWidth,
        eventMarkerInfoPopupFontSize = _ref.eventMarkerInfoPopupFontSize;

    var now = _moment2.default.utc();

    eventMarkerRadius = eventMarkerRadius || eventMarkerRadiusDefault;
    eventMarkerSpacingFromMidLine = eventMarkerSpacingFromMidLine || eventMarkerSpacingFromMidLineDefault;
    eventMarkerInfoPopupWidth = eventMarkerInfoPopupWidth || eventMarkerInfoPopupWidthDefault;
    eventMarkerInfoPopupHeight = eventMarkerInfoPopupHeight || eventMarkerInfoPopupHeightDefault;
    eventMarkerInfoPopupSpacingFromMarker = eventMarkerInfoPopupSpacingFromMarker || eventMarkerInfoPopupSpacingFromMarkerDefault;
    eventMarkerInfoPopupCaretWidth = eventMarkerInfoPopupCaretWidth || eventMarkerInfoPopupCaretWidthDefault;
    eventMarkerInfoPopupFontSize = eventMarkerInfoPopupFontSize || eventMarkerInfoPopupFontSizeDefault;

    // Get the graph's width from the bounding box of the midline.
    // This was using the svg's bounding box, but it would expand when data went off the end.
    var graphWidth = midLine.node().getBBox().width;

    // Construct a scale for drawing the time series, which converts from
    // [0, 1 minute ago] to [width, 0]. Ie, if the timedelta is 
    var timeScale = d3.scaleLinear().range([graphWidth, 0]).domain([0, graphDurationInMin * SECONDS_PER_MINUTE * MILLISECONDS_PER_SECOND]);

    // Calculate the locations of each marker to plot. Then:
    // 1. Calculate a timedelta between the current time and the time in the marker. This is used
    // instead of the timestamp as the time value because it changes as time marches forward, while
    // a tiemstamp wouldremain constant. Calulated by subtracting (current time) - (timestamp).
    // 2. Filter out all marks that render off the graph. This is jsut to be more efficient (why do
    // we need to render things that are off-screen?)
    var indicatorLocations = getIndicatorLocations(events).map(function (i) {
      return Object.assign({}, i, {
        timedelta: now.valueOf() - i.timestamp.valueOf()
      });
    }).filter(function (i) {
      return timeScale(i.timedelta) > -1 * eventMarkerInfoPopupWidth / 2;
    });

    // Render data in the chart.
    var dataSelection = dataGroup.selectAll('.real-time-capacity-point').data(indicatorLocations, function (d) {
      return d.timedelta;
    });

    // When new circles are created, plot them at their proper x and y coordinates.
    var dataEnterSelection = dataSelection.enter().append('g').attr('class', 'real-time-capacity-point');
    dataEnterSelection.append('circle');

    var dataEnterSelectionIndicator = dataEnterSelection.append('g').attr('class', 'real-time-capacity-info');
    dataEnterSelectionIndicator.append('rect').attr('class', 'real-time-capacity-info-content').attr('width', eventMarkerInfoPopupHeight).attr('height', eventMarkerInfoPopupHeight).attr('rx', 2).attr('ry', 2);
    dataEnterSelectionIndicator.append('text').attr('class', 'real-time-capacity-info-content-label');
    dataEnterSelectionIndicator.append('path').attr('class', 'real-time-capacity-info-pointer');

    // When circles are created or updated, plot ther position
    var dataMergeSelection = dataSelection.merge(dataEnterSelection);

    // Update the position of each circle.
    dataMergeSelection.select('circle').attr('r', eventMarkerRadius)
    // Subtract the radius to center the dot on the right edge of the chart.
    .attr('cx', function (d) {
      return timeScale(d.timedelta) - eventMarkerRadius;
    }).attr('cy', function (d) {
      if (d.count > 0) {
        // Render the dot above the midline.
        return cardHeightInPx / 2 - (eventMarkerRadius + eventMarkerSpacingFromMidLine);
      } else {
        // Render the dot below the midline.
        return cardHeightInPx / 2 + (eventMarkerRadius + eventMarkerSpacingFromMidLine);
      }
    }).attr('class', function (d) {
      return (0, _classnames2.default)('real-time-capacity-count-marker', d.count > 0 ? 'in' : 'out');
    });

    // Add info popups when the count is over 1 for a given column.
    dataMergeSelection.select('.real-time-capacity-info').attr('transform', function (d) {
      // Draw the info popup at the x coordinate that relates to the timestamp. Ensure the centers
      // of the event marker and the info popup are aligned by subtracting half the width of each
      // item from the x position.
      var xPos = timeScale(d.timedelta) - eventMarkerInfoPopupWidth / 2 - eventMarkerRadius;

      // Draw the info popup at the y coordinate that relates to if the event was an ingress or
      // egress.
      var yPos = void 0;
      if (d.count > 0) {
        // Render the popover above the midline.
        yPos = cardHeightInPx / 2 - (eventMarkerRadius * 2 + eventMarkerSpacingFromMidLine + eventMarkerInfoPopupSpacingFromMarker) - eventMarkerInfoPopupHeight;
      } else {
        // Render the popover below the midline.
        yPos = cardHeightInPx / 2 + (eventMarkerRadius * 2 + eventMarkerSpacingFromMidLine + eventMarkerInfoPopupSpacingFromMarker);
      }

      return 'translate(' + xPos + ',' + yPos + ')';
    }).attr('class', function (d) {
      return (0, _classnames2.default)('real-time-capacity-info', { hidden: Math.abs(d.count) === 1 });
    });

    dataMergeSelection.select('text').text(function (d) {
      return Math.abs(d.count);
    }).attr('font-size', eventMarkerInfoPopupFontSize).attr('transform', function (d) {
      // Center the text horizontally and vertically within the info popup
      var bbox = this.getBoundingClientRect();
      var textWidth = bbox.right - bbox.left;
      var textHeight = bbox.bottom - bbox.top + eventMarkerInfoPopupFontSize + 2;
      var centeredXPosition = (eventMarkerInfoPopupWidth - textWidth) / 2;
      var centeredYPosition = (eventMarkerInfoPopupHeight - textHeight) / 2;

      // add `eventMarkerInfoPopupHeight` to the y position to make the text be positioned from
      // the top of the container, not the bottom.
      return 'translate(' + centeredXPosition + ', ' + (centeredYPosition + eventMarkerInfoPopupHeight - sailecBaselineOffset) + ')';
    });

    var caretXPosition = (eventMarkerInfoPopupWidth - eventMarkerInfoPopupCaretWidth) / 2;
    dataMergeSelection.select('.real-time-capacity-info-pointer').attr('d', function (d) {
      if (d.count > 0) {
        // Downward-facing caret.
        return [
        // Move to the upper left corner of the caret.
        'M ' + caretXPosition + ' ' + eventMarkerInfoPopupHeight,
        // Move to the bottom point of the caret.
        'L ' + (caretXPosition + eventMarkerInfoPopupCaretWidth / 2) + ' ' + (eventMarkerInfoPopupHeight + eventMarkerInfoPopupCaretWidth / 2),
        // Move to the upper right of the caret.
        'L ' + (caretXPosition + eventMarkerInfoPopupCaretWidth) + ' ' + eventMarkerInfoPopupHeight].join(' ');
      } else {
        // Upward-facing caret.
        return [
        // Move to the lower left corner of the caret.
        'M ' + caretXPosition + ' 0',
        // Move to the top point of the caret.
        'L ' + (caretXPosition + eventMarkerInfoPopupCaretWidth / 2) + ' ' + -1 * eventMarkerInfoPopupCaretWidth / 2,
        // Move to the upper right of the caret.
        'L ' + (caretXPosition + eventMarkerInfoPopupCaretWidth) + ' 0'].join(' ');
      }
    });

    dataSelection.exit().remove();
  };
}

